/* $Id: io.c,v 1.1 1999/10/04 18:48:13 maxime Exp $ */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#include "libdev.h"
#include "local.h"

#define aux_on "\033[:#i"
#define aux_off "\033[:#f"
#define kb_off "\033[2h"
#define kb_on "\033[2l"

/*
 * open_port -      
 *  :
 *   pn -    
 *   baud - c
 *   bit -   : 8, 7  0 -  
 *   parity - : 'E' , 'N'   , 'O' , 0 -  
 *   stop_bit - 1, 2  0 - 
 *
 *   full_modem = 0 - - ;
 *              = 1 - 
 *                    
 *
 *  :
 *   old_stty -    
 *
 * Return:  file descriptor - OK
 *          -1 - 
 */
open_port(pn, baud, bit, parity, stop_bit, full_modem, old_stty)
uchar *pn;
speed_t baud;
int bit;
int parity;
int stop_bit;
int full_modem;
struct termios *old_stty;
{
  struct termios stty;
  int port;
  int over_tt;
  uchar buf[BUFSIZE];

  if (strncmp(local_port, pn, strlen(local_port))) over_tt = 0;
  else over_tt = 1;
#ifdef DEBUG
  fprintf(stderr, "Line: %s\n", over_tt ? "local" : "remote");
#endif

  if (!over_tt) {
    if ((port=open(pn, O_RDWR)) == -1) {
#ifdef DEBUG
        perror(pn);
#endif
        return(-1);
     }
  }
  else
    port = fileno(stdin);

  tcgetattr(port, old_stty); /*    */
  /*    a     */
  memcpy(&stty, old_stty, sizeof(stty));
  /*        ,  :    */
  stty.c_lflag = 0;
  stty.c_oflag = 0;
  stty.c_iflag = 0;
  stty.c_cc[VMIN] = 40;  /*   .  */
  stty.c_cc[VTIME] = 1; /*  = 0.1       */

  if (!over_tt) {
   switch (bit) {
    case 8: stty.c_cflag |= CS8;
            break;
    case 7: stty.c_cflag |= CS7;
            break;
    case 6: stty.c_cflag |= CS6;
            break;
    case 5: stty.c_cflag |= CS5;
            break;
    default: break;
   }
   switch (parity) {
    case 'O':
    case 'o':
              stty.c_iflag |= (INPCK);   /*  .    */
              stty.c_cflag |= PARENB | PARODD;
              break;
    case 'E':
    case 'e':
              stty.c_iflag |= (INPCK);   /*  .    */
              stty.c_cflag |= PARENB;
              stty.c_cflag &= ~PARODD;
              break;
     case 'N':
     case 'n':
              stty.c_iflag |= (INPCK);   /*  .    */
              stty.c_cflag &= ~PARENB;
              break;
     default:
              stty.c_iflag &= ~INPCK;   /*      */
              stty.c_cflag &= ~PARENB;
              break;
   }
   switch (stop_bit) {
     case 1:
              stty.c_cflag &= ~CSTOPB;
              break;
     case 2:
              stty.c_cflag |= CSTOPB;
              break;
     default: break;
   }
   if (full_modem) {
      stty.c_iflag &= ~( IXANY | IXON | IXOFF );   /*  XANY (   .)  */
      stty.c_cflag |= (CRTSCTS); /*   RTS/CTS */
      stty.c_cflag &= ~(CLOCAL);  /*     */
   }
   else {
      stty.c_iflag &= ~( IXANY | IXON | IXOFF );   /*  XANY (   .)  */
      stty.c_cflag &= ~(CRTSCTS); /*   RTS/CTS */
      stty.c_cflag |= (CLOCAL);   /*     */
   }
   cfsetospeed(&stty, baud);
   cfsetispeed(&stty, baud);
  }
  /*    stty- */
  tcsetattr(port, TCSANOW, &stty);
  if (over_tt) {
//   printf("%s%s", kb_off, aux_on);
     printf("%s", aux_on);
     fflush(stdout);
  }
#ifdef CLEAR_FIRST_INPUT
  read_box(port, buf, 0.05, NULL);       /*    */
#endif
  return(port);
}

close_port(port, old_stty)
int port;
struct termios *old_stty;
{
   int over_tt;

   if (over_tt = (port == fileno(stdin))) {
//   printf("%s%s", aux_off, kb_on);
     printf("%s", aux_off);
     fflush(stdout);
   }

   tcsetattr(port, TCSANOW, old_stty);
   if (!over_tt) close(port);
}

/* read_box() -    ,  
 *               ,  .
 *		  < 0 -  
 * Return: 0  - 
 *         -1 -   
 *         -2 -  /
 *         >0 -   
 */
int read_box(fp, buffer, timeout, interrupt_kb)
int fp;            /* File Descriptor          */
uchar *buffer;      /*            */
float timeout;     /*                           */
int (*interrupt_kb)(); /* ,      
                          .    
                          : 0 -  ;
                                              1 - .
                         NULL,      
                         
                      */
{
  int len = 0;

#ifdef SELECT
  int i;
  fd_set read_fd;
  struct timeval tv;
  double ffract, fint;
  int over_tt;

  over_tt = (fp == fileno(stdin));

  if (timeout >= 0.0) {
    ffract = modf(timeout, &fint);

    tv.tv_sec = ceil(fint);
    tv.tv_usec = ceil(ffract * 1000000);
  }

  FD_ZERO(&read_fd);

  do {
    FD_SET(fp, &read_fd);
    if ((!over_tt) && (interrupt_kb != NULL)) FD_SET(fileno(stdin), &read_fd);
    if (timeout < 0.0)
       i = select(FD_SETSIZE, &read_fd, NULL, NULL, NULL);
    else
       i = select(FD_SETSIZE, &read_fd, NULL, NULL, &tv);

    if (i == 0) return(0);      /* timeout */

    if ((!over_tt) &&
       (interrupt_kb != NULL) && /*   -   
                                       */
       (FD_ISSET(fileno(stdin), &read_fd)))  /*      */

       if (interrupt_kb()) {      /*  -       */
             return(-1);          /*  -      */
          }

  } while (!FD_ISSET(fp, &read_fd));
#endif

  len = read(fp, buffer, BUFSIZE);
  if (len < 0) {
     len = -2;
#ifdef DEBUG
     perror("Read error:");
  }
  for (i=0; i<len; i++)
      if (buffer[i] > '\040') fputc(buffer[i], stderr);
      else fprintf(stderr, " ^%c ", buffer[i]+64);
  fputc('\n', stderr);
#else
  }
#endif

  return(len);
}    

