/*$Id: mdmlock.c,v 1.1 1999/10/04 18:48:13 maxime Exp $
 *  libmodem library, a modem control facility.
 *
 *  mdmlock.c - modem lock files
 *
 *  Copyright (C) 1994, 1995  Riccardo Facchetti

+ adapted by Yura Kalinichenko for some own needs in March, 1999

 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  You can contact the author at this e-mail address:
 *
 *  riccardo@cdc8g5.cdc.polimi.it
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>

#ifndef BUFSIZE
#define BUFSIZE    128
#endif
#ifndef LOCKDIR
#define LOCKDIR "/var/lock"
#endif

/*
 * Signal to check if a PID exist
 */
#define SIGPIDEXIST	0


int mdm_lock(char *line) {
    char buffer1[BUFSIZE];
    char buffer[BUFSIZE];
    int fd;
    int lpid;
    int att;

/*
 * build the lockfile pathname
 */
    if (strcmp("local", line) == 0) return(0);    
    sprintf(buffer, "%s/LCK..%s", LOCKDIR, line);

/*
 * check if line is alredy locked
 */
    att = 0;
reopen_lock:
    if (att++ > 2) return(1);
    if ((fd = open(buffer, O_CREAT|O_WRONLY|O_EXCL)) == -1) {
/*
 * the lock exist. Get the pid and check if the process lpid exist.
 * If not, remove the stale lock
 */
        if ((fd = open (buffer, O_RDONLY)) == -1) {
           if (errno == ENOENT) goto reopen_lock;
           else return(1);
        }
        read(fd, buffer1, BUFSIZE);
        close(fd);
        sscanf(buffer1, "%10d", &lpid);

        if (kill(lpid, SIGPIDEXIST) == 0) {
            sleep(1);
            goto reopen_lock;
        }
        if ((unlink(buffer) == -1) && (errno != ENOENT)) return(1);
        goto reopen_lock;
    }

    fchmod(fd, 0666);
/*
 * lock the line
 */
    sprintf(buffer, "%10d\n", getpid());
    write(fd, buffer, strlen(buffer));
    close(fd);
    return(0);
}

int mdm_unlock(char *line) {
    char buffer[BUFSIZE];

    if (strcmp("local", line) == 0) return(0);    
/*
 * build the lockfile pathname
 */
    sprintf(buffer, "%s/LCK..%s", LOCKDIR, line);
    
    if (unlink(buffer) == -1)
        return(1);
    return (0);
}

int mdm_relock(char *line) {
    char buffer[BUFSIZE];
    int fd;

    if (strcmp("local", line) == 0) return(0);    
/*
 * build the lockfile pathname
 */
    sprintf(buffer, "%s/LCK..%s", LOCKDIR, line);

    if ((fd = open(buffer, O_CREAT|O_WRONLY)) == -1) return(1);
    fchmod(fd, 0666);
/*
 * lock the line
 */
    sprintf(buffer, "%10d\n", getpid());
    write(fd, buffer, strlen(buffer));
    close(fd);
    return(0);
}

