/* $Id: ves.c,v 1.1 1999/10/04 18:48:13 maxime Exp $ */
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include "libdev.h"
#include "local.h"

/*
 * ves -     
 * Return:
 *     1  -  
 *     0  -      
 *    -1  -    
 *    -2  -  /
 *    -3  -   
 *
 */
int ves (tty_name, attempts, timeout, ret_ves, kbhit)
uchar *tty_name; /*   ,     */
int attempts;   /* .                           */
double timeout; /* .                                       */
double *ret_ves;/* c                              */
int (*kbhit)(); /* ,      
                     .    
                     : 0 -  ;
                                         1 - .
                    NULL,      
                    
                  */
{
  int errflg;
  int l, r;
  int fp;
  double m;
  uchar buf[BUFSIZE];
  struct termios stty;
  
  *ret_ves = 0.0;
  r = 0;

  /*  -          */
  while (attempts-- != 0) {
#ifdef DEBUG
    fprintf(stderr, "Attempt: %d\n", attempts);
#endif    
    if (mdm_lock(tty_name)) return(-3);
    /*  ,     */
    if ((fp=open_port(tty_name, B9600, 8, 'N', 1, 0, &stty)) == -1) {
      /*   -   */
      mdm_unlock(tty_name);
      return(-3);
    }

    switch (init_scales(fp, timeout, kbhit))  { /*   */
    case 1:  /*  OK */
        break;
    case 0:  /* 0   () */
        close_port(fp, &stty);
        r = 0;
        continue;
    case -1: /*     */
        close_port(fp, &stty);
//        attempts = 0;
        r = -1;
        continue;
    case -2:
        r = -3;
        close_port(fp, &stty);
        continue;
    }

    switch (l=get_mass(fp, buf, timeout, kbhit)) { /*     */
    case 0:  /* 0   */
        r = 0;
        close_port(fp, &stty);
        show_error("  ");
        continue;
    case -1: /*     */
        r = -1;
        close_port(fp, &stty);
        attempts = 0;
        show_error("   ");
        continue;
    default: if (l < 0) { /*  - */
                r = -2;
                close_port(fp, &stty);
                show_error("   ");
                continue;
             }
             else {
               r = 1;
               break;  /*      */
             }
    }

    close_port(fp, &stty);
    errflg = 0;
    switch (test_mass(buf, l, &m)) {
    case 0:     /* OK */
    		r = 1;
                break;
    case 1:     /*    */
                show_error("  -   ...");
		r = 2;
                sleep(3);
                continue;
                break;
    case 2:     /*  */
                show_error("  !!!");
		r = 2;
                errflg++;
                break;
    case 3:     /*    -    */
                show_error("  ");
                r = 2;
                sleep(1);
                continue;
                break;
    default:    /*   */
                show_error(" ");
		r = 2;
                errflg++;
                break;
    }
    if (errflg) break;
    else {
      *ret_ves = m;
      break;
    }
 }
 mdm_unlock(tty_name);
 return(r);
}



/* test_mass -    .
 *
 * Return:
 *      0 -  ,  -  mass
 *      1 -   
 *      2 - 
 *      3 -   
 *      4 -  
 */
 int test_mass(block, length, mass)
 uchar *block;
 int length;
 double *mass;
 {
  register int i;
  double f;
  int sum;

#ifdef TEST_CSUM
  /*    */
  for (i=0,sum=0; i<15; i++) {
#ifdef DEBUG
    fprintf(stderr, "%#x ", block[i]);
#endif
    if ((i>3) && (i<12))
       sum = (sum + (block[i] & 0377)) & 177;
  }
  putchar('\n');
#ifdef DEBUG
  fprintf(stderr, "  = %x; : %x\n", block[12], sum);
#endif
  ;
  }
#endif

  /*    */
  if (block[2] == 'U') return(1);

  /*   */
  if (block[3] == 'F') return(2);
  /*   */
  if (((f = strtod(&block[4], NULL)) == HUGE_VAL) && errno) {
#ifdef DEBUG
     perror("    ");
#endif
     return(4);
  }
  if (block[3] == '-') *mass=-f;
  else *mass = f;
  return(0);
}

/*
 * get_mass -      
 * Return:
 *      0 -     
 *     -1 -   
 *     -2 -  /
 *     >0 -   
 */
int get_mass(fd, buf, timeout, kbhit)
int fd;         /* file descriptor   */
uchar *buf;
double timeout;
int (*kbhit)();
{
   int i;
   int over_tt;

   over_tt = (fd == fileno(stdin));

   buf[0] = DC1;
   if (write(fd, buf, 1) != 1) {
#ifdef DEBUG
     perror("Write DC1");
#endif
     return(-2);
   }
   i = read_box(fd, buf, timeout, kbhit);
   if (i > 0) {
      /*    */
      if ((i != 15) || (buf[0] != SOH) || (buf[1] != STX) ||
         (buf[13] != ETX) || (buf[14] != EOT)) {
#ifdef DEBUG
         fprintf(stderr, "  ");
#endif
         if (over_tt) { /*       */
              for (--i; i >= 0; i--)     /*     */
                 ungetc(buf[i], stdin);
                 /*  i == -1 */
         }
         else i = -2;   /*     .  i/o */
      }
  }

  return(i);
}

/*
 * init_scales -    
 *
 * Return: 1 - OK
 *         0 -       
 *        -1 -   
 *        -2 -  -
 *
 *
 */
init_scales(fd, timeout, kbhit)
int fd;         /* file descriptor   */
double timeout;
int (*kbhit)();
{
   uchar buf[10];
   int i;
   int over_tt;

   buf[0] = ENQ;
   if (write(fd, buf, 1) != 1) {  /*   ENQ */
#ifdef DEBUG
     perror("ENQ Write");
#endif
     return(-2);
   }

   over_tt = (fd == fileno(stdin));

   i=read_box(fd, buf, timeout, kbhit);  /*   */
   if ((i > 1) && over_tt) return(-1);
   if (i < 0) return(-2);
   if ((i == 1) && (buf[0] != ACK)) {         /*  = ACK ? */
#ifdef DEBUG
         fprintf(stderr, "ACK Read - bad ACK\n");
#endif
         if (over_tt) return(-1);
         else return(-2);
   }
#ifdef DEBUG
   else
     fprintf(stderr, "ACK Read: %d\n", i);
#endif
  return(i);    /*    */
}
